/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007-2008 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Activator.java 3659 2008-07-16 07:12:52Z fornacif $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.war.impl.osgi;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceRegistration;
import org.osgi.util.tracker.ServiceTracker;
import org.ow2.util.ee.metadata.war.api.IWarDeployableMetadataFactory;
import org.ow2.util.ee.metadata.war.impl.WarDeployableMetadataFactory;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.IScanner;

/**
 * Activator which add war deployable metadata factory to
 * DeployableMetadataManager.
 * @author Gael Lalire
 * @author Francois Fornaciari
 */
public class Activator implements BundleActivator, ServiceListener {

    /**
     * OSGi BundleContext.
     */
    private BundleContext bundleContext = null;

    /**
     * ServiceRegistration for the IScanner.
     */
    private ServiceRegistration serviceRegistration = null;

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(Activator.class);

    /**
     * Called when this bundle is started so the Framework can perform the
     * bundle-specific activities necessary to start this bundle. This method
     * can be used to register services or to allocate any resources that this
     * bundle needs.
     * @param context The execution context of the bundle being started.
     * @throws Exception
     */
    public void start(final BundleContext bc) throws Exception {
        this.bundleContext = bc;
        // Try to register the IWarDeployableMetadataFactory service
        registerService();
        // Add service listener to be notified when the IScanner
        // service is registered or is unregistering
        bc.addServiceListener(this, "(objectClass=" + IScanner.class.getName() + ")");
    }

    /**
     * Called when this bundle is stopped so the Framework can perform the
     * bundle-specific activities necessary to stop the bundle.
     * @param context The execution context of the bundle being stopped.
     */
    public void stop(final BundleContext bc) {
        unregisterService();
    }

    /**
     * @see org.osgi.framework.ServiceListener#serviceChanged(org.osgi.framework.ServiceEvent)
     */
    public void serviceChanged(final ServiceEvent serviceEvent) {
        // The IScanner service is registered
        if (serviceEvent.getType() == ServiceEvent.REGISTERED) {
            try {
                registerService();
            } catch (Exception e) {
                logger.error("Cannot register the " + IWarDeployableMetadataFactory.class.getName() + " service");
            }
        } else if (serviceEvent.getType() == ServiceEvent.UNREGISTERING) {
            // The IScanner service is unregistering, so
            // unregister the IWarDeployableMetadataFactory service
            unregisterService();
        }

    }

    /**
     * Register the IWarDeployableMetadataFactory service.
     * @throws Exception If the service registration fails
     */
    private void registerService() throws Exception {
        ServiceTracker serviceTracker = new ServiceTracker(bundleContext, bundleContext.createFilter("(objectClass="
                + IScanner.class.getName() + ")"), null);
        serviceTracker.open();
        IScanner scanner = (IScanner) serviceTracker.getService();
        if (scanner != null) {
            serviceRegistration = bundleContext.registerService(IWarDeployableMetadataFactory.class.getName(),
                    new WarDeployableMetadataFactory(scanner), null);
        }
    }

    /**
     * Unregister the IWarDeployableMetadataFactory service
     */
    private void unregisterService() {
        if (serviceRegistration != null) {
            serviceRegistration.unregister();
            serviceRegistration = null;
        }
    }

}
